@@ -3,7 +3,8 @@
 ## coefficients from model fits (clm()s) with nominal effects.
 
 getThetamat <-
-  function(terms, alpha, assign, contrasts, xlevels, tJac, dataClasses)
+  function(terms, alpha, assign, contrasts, tJac, xlevels,
+           dataClasses)
 ### Compute matrix of thresholds for all combinations of levels of
 ### factors in the nominal formula.
 ###
@@ -14,6 +15,8 @@ getThetamat <-
 ###   the nominal effects
 ### contrasts: list of contrasts for the nominal effects
 ### tJac: threshold Jacobian with appropriate dimnames.
+### xlevels: names of levels of factors among the nominal effects.
+### dataClasses: vector of data classes cf. help(.MFclass)
 ###
 ### Output:
 ### Theta: data.frame of thresholds
@@ -22,31 +25,32 @@ getThetamat <-
 {
     ## Make matrix of thresholds; Theta:
     Theta <- matrix(alpha, ncol=ncol(tJac), byrow=TRUE)
-    ## matrix with variables-by-terms:
+    ## Matrix with variables-by-terms:
     factor.table <- attr(terms, "factors")
     all.varnm <- rownames(factor.table)
 ### NOTE: need to index with all.varnm not to include (weights) and
 ### possibly other stuff.
     var.classes <- dataClasses[all.varnm]
-  numeric.var <- which(var.classes != "factor")
-### NOTE: for "dataClasses" see help(.MFclass). logical variables are
-### treated like numeric variables.
-  ## terms associated with numerical variables:
+    numeric.var <- which(var.classes != "factor")
+### NOTE: Logical variables are treated as numeric variables.
     numeric.terms <- factor.terms <- numeric(0)
     if(length(factor.table)) {
+        ## Terms associated with numerical variables:
         numeric.terms <-
             which(colSums(factor.table[numeric.var, , drop=FALSE]) > 0)
-        ## terms only involving factor variables:
+        ## Terms only involving factor variables:
         factor.terms <-
             which(colSums(factor.table[numeric.var, , drop=FALSE]) == 0)
     }
-    ## remove rows in Theta corresponding to numerical variables:
+    ## Remove rows in Theta for numeric variables:
     if(length(numeric.terms)) {
 ### NOTE: ncol(NOM) == length(asgn) == nrow(Theta)
 ### length(attr(terms, "term.labels")) == ncol(factor.table)
 ### NOTE: length(var.classes) == nrow(factor.table)
         numeric.rows <- which(assign %in% numeric.terms)
         Theta <- Theta[-numeric.rows, , drop=FALSE]
+        ## Drop terms so the design matrix, X for the factors does not
+        ## include numeric variables:
         if(length(factor.terms))
             terms <- drop.terms(terms, dropx=numeric.terms,
                                 keep.response=FALSE)
@@ -61,13 +65,20 @@ getThetamat <-
         mf.basic <- do.call(expand.grid, xlev)
         ## minimal complete design matrix:
         X <- model.matrix(terms, data=mf.basic,
-                          contrasts=contrasts)
-### NOTE: There are no contrasts for numerical variables.
-### FIXME: remove contrasts for 'ordered' variables.
-        ## from threshold parameters to thresholds:
-        Theta <- apply(Theta, 2, function(th) X %*% th)
+                          contrasts=contrasts[factor.varnm])
+### NOTE: There are no contrasts for numerical variables, but there
+### may be for ordered factors.
+        ## From threshold parameters to thresholds:
+### NOTE: some rows of Theta may contain NAs due to rank deficiency of
+### the NOM design matrix.
+        keep <- apply(Theta, 1, function(x) sum(is.na(x)) == 0)
+        ## Theta <- apply(Theta, 2, function(th) X %*% th)
+        tmp <- lapply(1:NCOL(Theta), function(i) {
+            X[, keep, drop=FALSE] %*% Theta[keep, i]
+        })
+        Theta <- do.call(cbind, tmp)
     }
-    ## adjust each row in Theta for threshold functions:
+    ## Adjust each row in Theta for threshold functions:
     tmp <- lapply(seq_len(nrow(Theta)), function(i)
                   c(tJac %*% Theta[i, ]))
     Theta <- do.call(rbind, tmp)
@@ -77,7 +88,7 @@ getThetamat <-
     colnames(Theta) <- rownames(tJac)
     res <- list(Theta = as.data.frame(Theta))
     ## add factor information if any:
-    if(nrow(Theta) > 1)  res$mf.basic <- mf.basic
+    if(NROW(Theta) > 1)  res$mf.basic <- mf.basic
     ## return:
     res
 }
